# Preferences.pm - The Object Class that provides a Preferences Object
# Created by James A. Pattie, 2005-03-22.

# Copyright (c) 2000-2005 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::UserProperties::Objects::Preferences;
use strict;
use Portal::Objects::Base;
use Portal::XML::ConfigParser;
use Portal::XML::ConfigObject;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

=head1 NAME

Preferences - Object used to build a preferences Object Class.

=head1 SYNOPSIS

  use Portal::UserProperties::Objects::Preferences;
  my $obj = Portal::UserProperties::Objects::Preferences->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Preferences is a preferences class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(methods, authObj, applicationObj, userObj, appName)

 Creates a new instance of the Portal::UserProperties::Objects::Preferences module.
 See Portal::Objects::Base(3) for a listing of required arguments.

 form 'main' defines the preferences that can be edited.
 form 'edit' defines the actual form items needed to edit the
   Portal's preferences.

 requires:
   methods - Portal::Methods instance
   authObj - Portal::Auth instance
   applicationObj - Portal::Application instance
   userObj - the user we are editing preferences for
   appName - the name of the App the preferences are being edited for.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( methods => undef, authObj => undef, applicationObj => undef, userObj => undef, appName => "",  @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  $self->{methods} = $args{methods};
  $self->{authObj} = $args{authObj};
  $self->{applicationObj} = $args{applicationObj};
  $self->{userObj} = $args{userObj};
  $self->{appName} = $args{appName};

  if (!$self->requiredArgsValid)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{objName} = "Preferences";
  $self->{data}->{main} =
    { "sidemenuLocation" => { -DataType => "string", -Value => "left", -PrefModule => "Global" },
    };

  $self->{usingForms} = 1;

  # the templates

  $self->{data}->{edit} =
    { "app" => { -Type => "hidden", -Value => "" },
      "state" => { -Type => "hidden", -Value => "" },
      "command" => { -Type => "hidden", -Value => "editAppPreferences" },
      "userId" => { -Type => "hidden", -Value => $self->userObj->get("id") },
      "uname" => { -Type => "hidden", -Value => $self->userObj->get("uname") },
      "appName" => { -Type => "hidden", -Value => $self->appName },
      "sidemenuLocation" => { -Type => "select", -Label => $self->langObj->map("sidemenuLocation"), -Value => "left", -Options => $self->variables->formSideMenuLocations() },
      "submitForm" => { -Type => "submit", -Value => $self->langObj->map("Update") },
      "formSubmitted" => { -Type => "hidden", -Value => 1 },
    };

  $self->{order}->{edit} = [ qw(sidemenuLocation) ];

  $self->{template}->{edit} = $self->formObj->createTemplate(data => $self->{data}->{edit}, class => "edit", rowsAlternate => 0,
    header => $self->langObj->map("EditAppUserPreferences"), order => $self->{order}->{edit});
  if ($self->formObj->error)
  {
    $self->error($self->formObj->errorMessage);
    return $self;
  }

  $self->{profile}->{edit} =
    { required => [ qw(app state command uname appName sidemenuLocation formSubmitted) ],
      optional => [ qw() ],
      constraints => {},
    };

  return $self;
}

=item bool requiredArgsValid

 Checks to make sure that the required args to new() were provided
 and appear to be valid.

=cut
sub requiredArgsValid
{
  my $self = shift;

  if (!defined $self->{methods})
  {
    $self->missing("methods");
  }
  elsif (!$self->{methods}->isValid)
  {
    $self->invalid("methods", $self->{methods}->errorMessage);
  }

  if (!defined $self->{authObj})
  {
    $self->missing("authObj");
  }
  elsif (!$self->{authObj}->isValid)
  {
    $self->invalid("authObj", $self->{authObj}->errorMessage);
  }

  if (!defined $self->{applicationObj})
  {
    $self->missing("applicationObj");
  }
  elsif (!$self->{applicationObj}->isValid)
  {
    $self->invalid("applicationObj", $self->{applicationObj}->errorMessage);
  }

  if (!defined $self->{userObj})
  {
    $self->missing("userObj");
  }
  elsif (!$self->{userObj}->isValid)
  {
    $self->invalid("userObj", $self->{userObj}->errorMessage);
  }

  if (length $self->{appName} == 0)
  {
    $self->missing("appName");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # make sure our required args are valid.
  if (!$self->requiredArgsValid)
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (!exists $self->variables->sideMenuLocations->{$self->get("sidemenuLocation")})
  {
    $self->invalid("sidemenuLocation", $self->get("sidemenuLocation"));
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item void setupEntry(scalar)

 Takes the preference name and populates the entry in the edit form.
 If the preference uses a select box to display itself, we build
 up the -Options array and populate it as needed.

 Ex: $obj->setEntry("sidemenuLocation");

=cut
sub setupEntry
{
  my $self = shift;
  my $pref = shift;

  if (! exists $self->{data}->{main}->{$pref})
  {
    $self->invalid("pref", "$pref", "Does not exist in main!");
    $self->error($self->genErrorString("invalid"));
    return;
  }

  # don't need to do anything since we have already populated the -Options.
}

=item bool validateEntry(scalar)

 Takes the preference name and validates that the user input in the
 formObj appears to be valid.  Does any data assembly required and
 updates the data->{main} entry for the specified preference.

 Returns 0 on error, 1 if everything is ok.

 Ex:
 if (!$obj->validateEntry("sidemenuLocation"))
 {
   $displayForm = 1;
 }

=cut
sub validateEntry
{
  my $self = shift;
  my $pref = shift;

  if (! exists $self->{data}->{main}->{$pref})
  {
    $self->invalid("pref", "$pref", "Does not exist in main!");
    $self->error($self->genErrorString("invalid"));
    return 0;
  }

  if ($pref eq "sidemenuLocation")
  {
    my $location = $self->formObj->getValidEntry($pref);
    if (! exists $self->variables->sideMenuLocations->{$location})
    {
      $self->formObj->invalid($pref, $location);
      $self->error($self->genErrorString("invalid"));
      return 0;
    }

    $self->set($pref, $location);
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Objects::Base(3)

=cut
